@@ -1,4 +1,4 @@
-# Copyright 2020 The DDSP Authors.
+# Copyright 2021 The DDSP Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,7 +18,7 @@
 
 import collections
 import copy
-from typing import Any, Dict, Text, TypeVar
+from typing import Any, Dict, Sequence, Text, TypeVar
 
 import gin
 import numpy as np
@@ -41,10 +41,27 @@ def make_iterable(x):
   """Wrap in a list if not iterable, return empty list if None."""
   if x is None:
     return []
+  elif isinstance(x, (np.ndarray, tf.Tensor)):
+    # Wrap in list so you don't iterate over the batch.
+    return [x]
   else:
     return x if isinstance(x, collections.Iterable) else [x]
 
 
+def to_dict(x, keys):
+  """Converts list to a dictionary with supplied keys."""
+  if isinstance(x, dict):
+    # No-op for dict.
+    return x
+  else:
+    # Wrap individual tensors in a list so we don't iterate over batch..
+    x = make_iterable(x)
+    if len(keys) != len(x):
+      raise ValueError(f'Keys: {keys} must be the same length as {x}')
+    # Use keys to create an output dictionary.
+    return dict(zip(keys, x))
+
+
 def copy_if_tf_function(x):
   """Copy if wrapped by tf.function.
 
@@ -59,6 +76,33 @@ def copy_if_tf_function(x):
   return copy.copy(x) if not tf.executing_eagerly() else x
 
 
+def nested_keys(nested_dict: Dict[Text, Any],
+                delimiter: Text = '/',
+                prefix: Text = '') -> Sequence[Text]:
+  """Returns a flattend list of nested key strings of a nested dict.
+
+  Args:
+    nested_dict: Nested dictionary.
+    delimiter: String that splits the nested keys.
+    prefix: Top-level key used for recursion, usually leave blank.
+
+  Returns:
+    List of nested key strings.
+  """
+  keys = []
+
+  for k, v in nested_dict.items():
+    key = k if not prefix else f'{prefix}{delimiter}{k}'
+
+    if not isinstance(v, dict):
+      keys.append(key)
+    else:
+      dict_keys = nested_keys(v, prefix=key)
+      keys += dict_keys
+
+  return keys
+
+
 def nested_lookup(nested_key: Text,
                   nested_dict: Dict[Text, Any],
                   delimiter: Text = '/') -> tf.Tensor:
@@ -77,7 +121,12 @@ def nested_lookup(nested_key: Text,
   # Return the nested value.
   value = nested_dict
   for key in keys:
-    value = value[key]
+    try:
+      value = value[key]
+    except KeyError:
+      raise KeyError(f'Key \'{key}\' as a part of nested key \'{nested_key}\' '
+                     'not found during nested dictionary lookup, out of '
+                     f'available keys: {nested_keys(nested_dict)}')
   return value
 
 
@@ -108,11 +157,11 @@ def safe_divide(numerator, denominator, eps=1e-7):
 
 def safe_log(x, eps=1e-5):
   """Avoid taking the log of a non-positive number."""
-  safe_x = tf.where(x <= 0.0, eps, x)
+  safe_x = tf.where(x <= eps, eps, x)
   return tf.math.log(safe_x)
 
 
-def logb(x, base=2.0, safe=False):
+def logb(x, base=2.0, safe=True):
   """Logarithm with base as an argument."""
   if safe:
     return safe_divide(safe_log(x), safe_log(base))
@@ -618,12 +667,15 @@ def harmonic_to_sinusoidal(harm_amp, harm_dist, f0_hz, sample_rate=16000):
   n_harmonics = int(harm_dist.shape[-1])
   freqs = get_harmonic_frequencies(f0_hz, n_harmonics)
   # Double check to remove anything above Nyquist.
-  harm_dist = remove_above_nyquist(f0_hz, harm_dist, sample_rate)
+  harm_dist = remove_above_nyquist(freqs, harm_dist, sample_rate)
+  # Renormalize after removing above nyquist.
+  harm_dist_sum = tf.reduce_sum(harm_dist, axis=-1, keepdims=True)
+  harm_dist = safe_divide(harm_dist, harm_dist_sum)
   amps = harm_amp * harm_dist
   return amps, freqs
 
 
-# Additive Synthesizer ---------------------------------------------------------
+# Harmonic Synthesizer ---------------------------------------------------------
 # TODO(jesseengel): Remove reliance on global injection for angular cumsum.
 @gin.configurable
 def angular_cumsum(angular_frequency, chunk_size=1000):
@@ -1093,15 +1145,21 @@ def fft_convolve(audio: tf.Tensor,
   """
   audio, impulse_response = tf_float32(audio), tf_float32(impulse_response)
 
+  # Get shapes of audio.
+  batch_size, audio_size = audio.shape.as_list()
+
   # Add a frame dimension to impulse response if it doesn't have one.
   ir_shape = impulse_response.shape.as_list()
   if len(ir_shape) == 2:
     impulse_response = impulse_response[:, tf.newaxis, :]
-    ir_shape = impulse_response.shape.as_list()
 
-  # Get shapes of audio and impulse response.
+  # Broadcast impulse response.
+  if ir_shape[0] == 1 and batch_size > 1:
+    impulse_response = tf.tile(impulse_response, [batch_size, 1, 1])
+
+  # Get shapes of impulse response.
+  ir_shape = impulse_response.shape.as_list()
   batch_size_ir, n_ir_frames, ir_size = ir_shape
-  batch_size, audio_size = audio.shape.as_list()
 
   # Validate that batch sizes match.
   if batch_size != batch_size_ir:
@@ -1239,7 +1297,10 @@ def sinc(x, threshold=1e-20):
   return tf.sin(x) / x
 
 
-def sinc_impulse_response(cutoff_frequency, window_size=512, sample_rate=None):
+def sinc_impulse_response(cutoff_frequency,
+                          window_size=512,
+                          sample_rate=None,
+                          high_pass=False):
   """Get a sinc impulse response for a set of low-pass cutoff frequencies.
 
   Args:
@@ -1249,6 +1310,8 @@ def sinc_impulse_response(cutoff_frequency, window_size=512, sample_rate=None):
       range [0, 1.0]. Shape [batch_size, n_time, 1].
     window_size: Size of the Hamming window to apply to the impulse.
     sample_rate: Optionally provide the sample rate.
+    high_pass: If true, filter removes frequencies below cutoff (high-pass), if
+      false [default], filter removes frequencies above cutoff (low-pass).
 
   Returns:
     impulse_response: A series of impulse responses. Shape
@@ -1273,7 +1336,16 @@ def sinc_impulse_response(cutoff_frequency, window_size=512, sample_rate=None):
   impulse_response = window * tf.math.real(impulse_response)
 
   # Normalize for unity gain.
-  impulse_response /= tf.reduce_sum(impulse_response, axis=-1, keepdims=True)
+  impulse_response /= tf.abs(
+      tf.reduce_sum(impulse_response, axis=-1, keepdims=True))
+
+  if high_pass:
+    # Invert filter.
+    pass_through = np.zeros_like(impulse_response)
+    pass_through[..., half_size] = 1.0
+    pass_through = tf.convert_to_tensor(pass_through, dtype=tf.float32)
+    impulse_response = pass_through - impulse_response
+
   return impulse_response
 
 
@@ -1311,7 +1383,8 @@ def sinc_filter(audio: tf.Tensor,
                 cutoff_frequency: tf.Tensor,
                 window_size: int = 512,
                 sample_rate: int = None,
-                padding: Text = 'same') -> tf.Tensor:
+                padding: Text = 'same',
+                high_pass: bool = False) -> tf.Tensor:
   """Filter audio with sinc low-pass filter.
 
   Args:
@@ -1326,6 +1399,8 @@ def sinc_filter(audio: tf.Tensor,
       same size as the input audio (audio_timesteps). For 'valid' the audio is
       extended to include the tail of the impulse response (audio_timesteps +
       window_size - 1).
+    high_pass: If true, filter removes frequencies below cutoff (high-pass), if
+      false [default], filter removes frequencies above cutoff (low-pass).
 
   Returns:
     Filtered audio. Tensor of shape
@@ -1334,5 +1409,6 @@ def sinc_filter(audio: tf.Tensor,
   """
   impulse_response = sinc_impulse_response(cutoff_frequency,
                                            window_size=window_size,
-                                           sample_rate=sample_rate)
+                                           sample_rate=sample_rate,
+                                           high_pass=high_pass)
   return fft_convolve(audio, impulse_response, padding=padding)
