@@ -1,4 +1,4 @@
-# Copyright 2020 The DDSP Authors.
+# Copyright 2021 The DDSP Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,8 +16,10 @@
 """Library of loss functions."""
 
 import functools
+from typing import Dict, Text
 
 import crepe
+from ddsp import dags
 from ddsp import spectral_ops
 from ddsp.core import hz_to_midi
 from ddsp.core import safe_divide
@@ -31,6 +33,70 @@ import tensorflow_probability as tfp
 tfd = tfp.distributions
 tfkl = tf.keras.layers
 
+# Define Types.
+TensorDict = Dict[Text, tf.Tensor]
+
+
+# ---------------------- Base Classes ------------------------------------------
+class Loss(tfkl.Layer):
+  """Base class. Duck typing: Losses just must implement get_losses_dict()."""
+
+  def get_losses_dict(self, *args, **kwargs):
+    """Returns a dictionary of losses for the model."""
+    loss = self(*args, **kwargs)
+    return {self.name: loss}
+
+
+@gin.register
+class LossGroup(dags.DAGLayer):
+  """Compute a group of loss layers on an outputs dictionary."""
+
+  def __init__(self, dag: dags.DAG, **kwarg_losses):
+    """Constructor, completely configurable via gin.
+
+    Args:
+      dag: A list of loss names/instances, with keys to extract call() inputs
+        from a dictionary, ex:
+
+        ['module', ['input_key', ...]]
+
+        'module': Loss module instance or string name of module. For example,
+          'spectral_loss' would access the attribute `loss_group.spectral_loss`.
+        'input_key': List of strings, nested keys in dictionary of dag outputs.
+
+      **kwarg_losses: Losses to add to LossGroup. Each kwarg that is a Loss will
+        be added as a property of the layer, so that it will be accessible as
+        `loss_group.kwarg`. Also, other keras kwargs such as 'name' are split
+        off before adding modules.
+    """
+    super().__init__(dag, **kwarg_losses)
+    self.loss_names = self.module_names
+
+  @property
+  def losses(self):
+    """Loss getter."""
+    return [getattr(self, name) for name in self.loss_names]
+
+  def call(self, outputs: TensorDict, **kwargs) -> TensorDict:
+    """Get a dictionary of loss values from all the losses.
+
+    Args:
+      outputs: A dictionary of model output tensors to feed into the losses.
+      **kwargs: Other kwargs for all the modules in the dag.
+
+    Returns:
+      A flat dictionary of losses {name: scalar}.
+    """
+    dag_outputs = super().call(outputs, **kwargs)
+    loss_outputs = {}
+    for k in self.loss_names:
+      loss_outputs.update(dag_outputs[k])
+    return loss_outputs
+
+  def get_losses_dict(self, outputs, **kwargs):
+    """Returns a dictionary of losses for the model, alias __call__."""
+    return self(outputs, **kwargs)
+
 
 # ---------------------- Losses ------------------------------------------------
 def mean_difference(target, value, loss_type='L1', weights=None):
@@ -63,7 +129,7 @@ def mean_difference(target, value, loss_type='L1', weights=None):
 
 
 @gin.register
-class SpectralLoss(tfkl.Layer):
+class SpectralLoss(Loss):
   """Multi-scale spectrogram loss.
 
   This loss is the bread-and-butter of comparing two audio signals. It offers
@@ -125,8 +191,7 @@ class SpectralLoss(tfkl.Layer):
       spectrogram_op = functools.partial(spectral_ops.compute_mag, size=size)
       self.spectrogram_ops.append(spectrogram_op)
 
-  def call(self, target_audio, audio):
-
+  def call(self, target_audio, audio, weights=None):
     loss = 0.0
 
     diff = spectral_ops.diff
@@ -139,41 +204,41 @@ class SpectralLoss(tfkl.Layer):
 
       # Add magnitude loss.
       if self.mag_weight > 0:
-        loss += self.mag_weight * mean_difference(target_mag, value_mag,
-                                                  self.loss_type)
+        loss += self.mag_weight * mean_difference(
+            target_mag, value_mag, self.loss_type, weights=weights)
 
       if self.delta_time_weight > 0:
         target = diff(target_mag, axis=1)
         value = diff(value_mag, axis=1)
         loss += self.delta_time_weight * mean_difference(
-            target, value, self.loss_type)
+            target, value, self.loss_type, weights=weights)
 
       if self.delta_freq_weight > 0:
         target = diff(target_mag, axis=2)
         value = diff(value_mag, axis=2)
         loss += self.delta_freq_weight * mean_difference(
-            target, value, self.loss_type)
+            target, value, self.loss_type, weights=weights)
 
       # TODO(kyriacos) normalize cumulative spectrogram
       if self.cumsum_freq_weight > 0:
         target = cumsum(target_mag, axis=2)
         value = cumsum(value_mag, axis=2)
         loss += self.cumsum_freq_weight * mean_difference(
-            target, value, self.loss_type)
+            target, value, self.loss_type, weights=weights)
 
       # Add logmagnitude loss, reusing spectrogram.
       if self.logmag_weight > 0:
         target = spectral_ops.safe_log(target_mag)
         value = spectral_ops.safe_log(value_mag)
-        loss += self.logmag_weight * mean_difference(target, value,
-                                                     self.loss_type)
+        loss += self.logmag_weight * mean_difference(
+            target, value, self.loss_type, weights=weights)
 
     if self.loudness_weight > 0:
       target = spectral_ops.compute_loudness(target_audio, n_fft=2048,
                                              use_tf=True)
       value = spectral_ops.compute_loudness(audio, n_fft=2048, use_tf=True)
-      loss += self.loudness_weight * mean_difference(target, value,
-                                                     self.loss_type)
+      loss += self.loudness_weight * mean_difference(
+          target, value, self.loss_type, weights=weights)
 
     return loss
 
@@ -182,7 +247,7 @@ class SpectralLoss(tfkl.Layer):
 # Peceptual Losses
 # ------------------------------------------------------------------------------
 @gin.register
-class EmbeddingLoss(tfkl.Layer):
+class EmbeddingLoss(Loss):
   """Embedding loss for a given pretrained model.
 
   Using these "perceptual" loss functions will help encourage better matching
@@ -259,7 +324,7 @@ class PretrainedCREPE(tfkl.Layer):
                activation_layer='conv5-maxpool',
                name='pretrained_crepe',
                trainable=False):
-    super(PretrainedCREPE, self).__init__(name=name, trainable=trainable)
+    super().__init__(name=name, trainable=trainable)
     self._model_capacity = model_capacity
     self._activation_layer = activation_layer
     spectral_ops.reset_crepe()
@@ -346,7 +411,7 @@ def freq_loss(f_hz, f_hz_target, loss_type='L1', weights=None):
 
 
 @gin.register
-class FilteredNoiseConsistencyLoss(tfkl.Layer):
+class FilteredNoiseConsistencyLoss(Loss):
   """Consistency loss for synthesizer controls.
 
   EXPERIMENTAL
@@ -363,7 +428,7 @@ class FilteredNoiseConsistencyLoss(tfkl.Layer):
 
 
 @gin.register
-class HarmonicConsistencyLoss(tfkl.Layer):
+class HarmonicConsistencyLoss(Loss):
   """Consistency loss for synthesizer controls.
 
   EXPERIMENTAL
@@ -414,7 +479,7 @@ class HarmonicConsistencyLoss(tfkl.Layer):
 # Sinusoidal Consistency Losses
 # ------------------------------------------------------------------------------
 @gin.register
-class WassersteinConsistencyLoss(tfkl.Layer):
+class WassersteinConsistencyLoss(Loss):
   """Compare similarity of two traces of sinusoids using wasserstein distance.
 
   EXPERIMENTAL
@@ -519,7 +584,7 @@ def wasserstein_distance(u_values, v_values, u_weights, v_weights, p=1.0):
 
 
 @gin.register
-class KDEConsistencyLoss(tfkl.Layer):
+class KDEConsistencyLoss(Loss):
   """Compare similarity of two traces of sinusoids using kernels.
 
   EXPERIMENTAL
@@ -649,7 +714,7 @@ class KDEConsistencyLoss(tfkl.Layer):
 # Differentiable Two-way Mismatch Loss
 # ------------------------------------------------------------------------------
 @gin.register
-class TWMLoss(tfkl.Layer):
+class TWMLoss(Loss):
   """Two-way Mismatch, encourages sinusoids to be harmonics best f0 candidate.
 
   EXPERIMENTAL
