@@ -1,4 +1,4 @@
-# Copyright 2020 The DDSP Authors.
+# Copyright 2021 The DDSP Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,7 +23,7 @@ import librosa
 import numpy as np
 import tensorflow.compat.v2 as tf
 
-_CREPE_SAMPLE_RATE = 16000
+CREPE_SAMPLE_RATE = 16000
 _CREPE_FRAME_SIZE = 1024
 
 F0_RANGE = 127.0  # MIDI
@@ -32,8 +32,13 @@ LD_RANGE = 120.0  # dB
 
 def stft(audio, frame_size=2048, overlap=0.75, pad_end=True):
   """Differentiable stft in tensorflow, computed in batch."""
-  audio = tf_float32(audio)
   assert frame_size * overlap % 2.0 == 0.0
+
+  # Remove channel dim if present.
+  audio = tf_float32(audio)
+  if len(audio.shape) == 3:
+    audio = tf.squeeze(audio, axis=-1)
+
   s = tf.signal.stft(
       signals=audio,
       frame_length=int(frame_size),
@@ -144,12 +149,13 @@ def diff(x, axis=-1):
     ValueError: Axis out of range for tensor.
   """
   shape = x.shape.as_list()
-  if axis >= len(shape):
+  ndim = len(shape)
+  if axis >= ndim:
     raise ValueError('Invalid axis index: %d for tensor with only %d axes.' %
-                     (axis, len(shape)))
+                     (axis, ndim))
 
-  begin_back = [0 for _ in range(len(shape))]
-  begin_front = [0 for _ in range(len(shape))]
+  begin_back = [0 for _ in range(ndim)]
+  begin_front = [0 for _ in range(ndim)]
   begin_front[axis] = 1
 
   shape[axis] -= 1
@@ -160,7 +166,7 @@ def diff(x, axis=-1):
 
 
 def amplitude_to_db(amplitude, use_tf=False):
-  """Converts amplitude to dB."""
+  """Converts amplitude to decibels."""
   lib = tf if use_tf else np
   log10 = (lambda x: tf.math.log(x) / tf.math.log(10.0)) if use_tf else np.log10
   amin = 1e-20  # Avoid log(0) instabilities.
@@ -169,6 +175,11 @@ def amplitude_to_db(amplitude, use_tf=False):
   return db
 
 
+def db_to_amplitude(db):
+  """Converts decibels to amplitude."""
+  return 10.0**(db / 20.0)
+
+
 @gin.register
 def compute_loudness(audio,
                      sample_rate=16000,
@@ -297,7 +308,8 @@ def compute_rms_energy(audio,
                        frame_rate=250,
                        frame_size=2048):
   """Compute root mean squared energy of audio."""
-  n_secs = len(audio) / float(sample_rate)  # `n_secs` can have milliseconds
+  n_samples = audio.shape[0] if len(audio.shape) == 1 else audio.shape[1]
+  n_secs = n_samples / float(sample_rate)  # `n_secs` can have milliseconds
   expected_len = int(n_secs * frame_rate)
 
   audio = tf_float32(audio)
