@@ -1,4 +1,4 @@
-# Copyright 2020 The DDSP Authors.
+# Copyright 2021 The DDSP Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,7 +24,7 @@ The pip install installs a `ddsp_run` script that can be called directly.
 ddsp_run \
 --mode=train \
 --alsologtostderr \
---save_dir=~/tmp/$USER-ddsp-0 \
+--save_dir=/tmp/$USER-ddsp-0 \
 --gin_file=models/ae.gin \
 --gin_file=datasets/nsynth.gin \
 --gin_param=batch_size=16
@@ -36,13 +36,13 @@ For evaluation and sampling, only the dataset file is required.
 ddsp_run \
 --mode=eval \
 --alsologtostderr \
---save_dir=~/tmp/$USER-ddsp-0 \
+--save_dir=/tmp/$USER-ddsp-0 \
 --gin_file=datasets/nsynth.gin
 
 ddsp_run \
 --mode=sample \
 --alsologtostderr \
---save_dir=~/tmp/$USER-ddsp-0 \
+--save_dir=/tmp/$USER-ddsp-0 \
 --gin_file=datasets/nsynth.gin
 
 
@@ -53,7 +53,7 @@ datasets used for a paper's experiments, so only require one gin file to train.
 ddsp_run \
 --mode=train \
 --alsologtostderr \
---save_dir=~/tmp/$USER-ddsp-0 \
+--save_dir=/tmp/$USER-ddsp-0 \
 --gin_file=papers/iclr2020/nsynth_ae.gin
 
 
@@ -65,6 +65,7 @@ import time
 from absl import app
 from absl import flags
 from absl import logging
+from ddsp.training import cloud
 from ddsp.training import eval_util
 from ddsp.training import models
 from ddsp.training import train_util
@@ -79,24 +80,33 @@ FLAGS = flags.FLAGS
 # Program flags.
 flags.DEFINE_enum('mode', 'train', ['train', 'eval', 'sample'],
                   'Whether to train, evaluate, or sample from the model.')
-flags.DEFINE_string('save_dir', '~/tmp/ddsp',
+flags.DEFINE_string('save_dir', '/tmp/ddsp',
                     'Path where checkpoints and summary events will be saved '
                     'during training and evaluation.')
 flags.DEFINE_string('restore_dir', '',
                     'Path from which checkpoints will be restored before '
                     'training. Can be different than the save_dir.')
 flags.DEFINE_string('tpu', '', 'Address of the TPU. No TPU if left blank.')
-flags.DEFINE_multi_string('gpu', [],
-                          'Addresses of GPUs for sync data-parallel training.'
-                          'Only needs to be specified for using multiple GPUs.')
+flags.DEFINE_string('cluster_config', '',
+                    'Worker-specific JSON string for multiworker setup. '
+                    'For more information see train_util.get_strategy().')
 flags.DEFINE_boolean('allow_memory_growth', False,
                      'Whether to grow the GPU memory usage as is needed by the '
                      'process. Prevents crashes on GPUs with smaller memory.')
+flags.DEFINE_boolean('hypertune', False,
+                     'Enable metric reporting for hyperparameter tuning, such '
+                     'as on Google Cloud AI-Platform.')
+flags.DEFINE_float('early_stop_loss_value', None,
+                   'Stops training early when the `total_loss` reaches below '
+                   'this value during training.')
 
 # Gin config flags.
 flags.DEFINE_multi_string('gin_search_path', [],
                           'Additional gin file search paths.')
-flags.DEFINE_multi_string('gin_file', [], 'List of paths to the config files.')
+flags.DEFINE_multi_string('gin_file', [],
+                          'List of paths to the config files. If file '
+                          'in gstorage bucket specify whole gstorage path: '
+                          'gs://bucket-name/dir/in/bucket/file.gin.')
 flags.DEFINE_multi_string('gin_param', [],
                           'Newline separated list of Gin parameter bindings.')
 
@@ -128,16 +138,20 @@ def parse_gin(restore_dir):
     use_tpu = bool(FLAGS.tpu)
     opt_default = 'base.gin' if not use_tpu else 'base_tpu.gin'
     gin.parse_config_file(os.path.join('optimization', opt_default))
+    eval_default = 'eval/basic.gin'
+    gin.parse_config_file(eval_default)
 
     # Load operative_config if it exists (model has already trained).
     operative_config = train_util.get_latest_operative_config(restore_dir)
     if tf.io.gfile.exists(operative_config):
       logging.info('Using operative config: %s', operative_config)
+      operative_config = cloud.make_file_paths_local(operative_config, GIN_PATH)
       gin.parse_config_file(operative_config, skip_unknown=True)
 
     # User gin config and user hyperparameters from flags.
+    gin_file = cloud.make_file_paths_local(FLAGS.gin_file, GIN_PATH)
     gin.parse_config_files_and_bindings(
-        FLAGS.gin_file, FLAGS.gin_param, skip_unknown=True)
+        gin_file, FLAGS.gin_param, skip_unknown=True)
 
 
 def allow_memory_growth():
@@ -164,13 +178,15 @@ def main(unused_argv):
 
   gfile.makedirs(restore_dir)  # Only makes dirs if they don't exist.
   parse_gin(restore_dir)
+  train_util.gin_register_keras_layers()
 
   if FLAGS.allow_memory_growth:
     allow_memory_growth()
 
   # Training.
   if FLAGS.mode == 'train':
-    strategy = train_util.get_strategy(tpu=FLAGS.tpu, gpus=FLAGS.gpu)
+    strategy = train_util.get_strategy(tpu=FLAGS.tpu,
+                                       cluster_config=FLAGS.cluster_config)
     with strategy.scope():
       model = models.get_model()
       trainer = trainers.Trainer(model, strategy)
@@ -178,7 +194,9 @@ def main(unused_argv):
     train_util.train(data_provider=gin.REQUIRED,
                      trainer=trainer,
                      save_dir=save_dir,
-                     restore_dir=restore_dir)
+                     restore_dir=restore_dir,
+                     early_stop_loss_value=FLAGS.early_stop_loss_value,
+                     report_loss_to_hypertune=FLAGS.hypertune)
 
   # Evaluation.
   elif FLAGS.mode == 'eval':
